function kinetics_batch_analysis

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is written by Pengning Xu in 06/2019
% based on the prototype of Jacob Gauer in Dorothy Erie Lab 
% at UNC Chapel Hill
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This program will analyze all of the .traces files in a given folder and
% output a .txt file for each molecule containing the several columns of
% data, including the raw and smoothed data, a FRET histogram, a list of
% data identified as bad, and the results of two edge detection methods.

% Opening the folder containing the .traces files.
[FileName1,PathName] = uigetfile('*.txt','Select the starting trace file.');
[FileName2,~] = uigetfile('*.txt','Select the final trace file.');
firstmovie = strrep(strrep(FileName1,'trace_',''),'.txt','');
lastmovie = strrep(strrep(FileName2,'trace_',''),'.txt','');
startnum = str2num(firstmovie);
endnum = str2num(lastmovie);


% Experiment Parameters
prompstr={'Specify traces starttime, endtime and timestep(second)' ...
          'Specify leakage factor', ...
          'Specify gamma factor'};
initstr={'0.4 300 0.2','0','1'}; 
titlestr='Experiment Factors';
nlines=1;
experiment=inputdlg(prompstr,titlestr,nlines,initstr);
frameproperty = str2num(experiment{1});
starttime = frameproperty(1,1);
endtime = frameproperty(1,2);
timestep = frameproperty(1,3);
leakage = str2num(experiment{2});
gamma = str2num(experiment{3});


% Choose Algorithms 
prompstr={'Do you want to apply Chung-Kennedy Filter?(Y/N)', ...
          'Do you want to apply Gaussian Kernel Edge Detection?(Y/N)', ...
          'Do you want to apply Chung-Kennedy Edge Detection?(Y/N)'};
initstr={'N','N','Y'}; 
titlestr='Choose Algorithms';
nlines=1;
result0=inputdlg(prompstr,titlestr,nlines,initstr);


% Chung-Kennedy Filter Dialog Box
if result0{1} == 'Y'
    prompstr={'Windows','Predictor Windows','Exponents'};
    initstr={'2 4 8','4','10'}; 
    titlestr='CK Filter Parameters';
    nlines=1;
    result1=inputdlg(prompstr,titlestr,nlines,initstr);
    % Chung Kennedy Filter Parameters
    WW = str2num(result1{1});        % window size(s) for predictors
    MM = str2num(result1{2});        % window for comparison
    PP = str2num(result1{3});        % exponent for making the forward and backward predictors
end

% Gaussian Kernel Edge Detection Dialog Box
if result0{2} == 'Y'
    prompstr={'Thresholds','Scales','Transition Width'};
    initstr={'0.2 0.3 0.4 0.5 0.6','1 2 4 8','2'}; 
    titlestr='GK Edge Parameters';
    nlines=1;
    result2=inputdlg(prompstr,titlestr,nlines,initstr);
end

% Chung - Kennedy Edge Detection Dialog Box
if result0{3} == 'Y'
    prompstr={'Windows','Percentile','Separation'};
    initstr={'4','90','1'}; 
    titlestr='CK Edge Parameters';
    nlines=1;
    result3=inputdlg(prompstr,titlestr,nlines,initstr);
end

directory=[PathName, '/'];
disp(directory)
cd(directory);  
close all;

% Looping through all .traces files (each movie).
for movienum = startnum:endnum			
    file = ['trace_' num2str(movienum)];
    to_open = strcat(file,'.txt');
    
    if exist(to_open) == 2
        close all;
        Ntraces=2;
        raw=importdata(to_open);
        len = length(raw.data);
        disp('Done reading data.');
        
        % Setting up matrices to be used later.
        time1 = zeros(1,len);
        Data=zeros(Ntraces+1,len);
        donor=zeros(Ntraces/2,len);
        acceptor=zeros(Ntraces/2,len);
        time_1 = zeros(len,1);
        donor_1 = zeros(len,Ntraces/2);
        acceptor_1 = zeros(len,Ntraces/2);
        donorraw = zeros(len,Ntraces/2);
        acceptorraw = zeros(len,Ntraces/2);
        fret_smth = zeros(len,Ntraces/2);
        fretraw = zeros(len,Ntraces/2);
        GKtransitions = zeros(len,Ntraces/2); GKtransindex = [];
        CKtransitions = zeros(len,Ntraces/2); CKtransindex = [];



        % Counters used to go forward and backward when sorting
        molnumber=1;


        % Looping through all molecules
        while molnumber < (Ntraces/2 + 1) 
            disp(['   Working on molecule ' num2str(molnumber) ' of ' num2str(Ntraces/2) ' in cascade ' num2str(movienum) '.'])
            
            % Applying correction factors
            time_1 = raw.data(:,1);
            donor_1 = raw.data(:,2);                    % Raw donor signal is corrected with the gamma factor
            acceptor_1 = raw.data(:,3);	% Raw acceptor signal is corrected with the leakage factor
            donorraw=donor_1;
            acceptorraw=acceptor_1;
    
            % Applying the Chung-Kennedy filter            
            if result0{1} == 'Y'
                disp(['      Smoothing the data using the Chung-Kennedy filter.'])
                dfilterme = donor_1(:,1);     
                afilterme = acceptor_1(:,1);
                donorraw=donor_1(:,1);
                acceptorraw=acceptor_1(:,1);
                [doutput, aoutput] = FRET_ck(dfilterme, afilterme, WW, MM, PP);
                donor_1 = doutput{3};
                acceptor_1 = aoutput{3};
            else
                donorraw = donor_1(:,1);
                acceptorraw = acceptor_1(:,1);
            end
    
            % Calculating FRET.
            if length(acceptor_1) > length(donor_1)
                len = length(donor_1);
            else
                len = length(acceptor_1);
            end

            % Identifying bleaches, blinks, and intensities that are too high or too low.
            disp(['      Checking for bad data.'])
            databad = 0;
            databad = baddata(donorraw, acceptorraw, donor_1, acceptor_1);
            databad = zeros(1,len);
            
            
            % If the whole molecule is bad, skip to the next one.
            if isequal(databad,ones(1,len))==1
                disp(['      * * * Molecule ' num2str(molnumber) ' was entirely unusable. * * *'])
                molnumber=molnumber+1;

            % Ensure 0 < FRET < 1 and make FRET = 0 at bleaches and blinks.
            else
                fretres = zeros(1,len);
                disp(['      Calculating FRET.'])                   
                for t = 1:len
                    ft(t)=0;
                    if databad(t) ==1
                        fret_smth(t) = 0;
                        fretraw(t) = 0;
                    else
                        fret_smth(t) = acceptor_1(t)/(donor_1(t)+acceptor_1(t));
                        fretraw(t) = acceptorraw(t)/(donorraw(t)+acceptorraw(t));
                        fretres(t)= fretraw(t)-fret_smth(t);
                        if  fret_smth(t) > 0.9999
                            fret_smth(t) = 0.9999;
                        elseif fret_smth(t) < 0
                            fret_smth(t) = 0.0001;
                        end
                    end
                end
            
                % Making a histogram of FRET at each timepoint, removing zeros and ones.
                histogram=0; xout=0; count=0; hist_ctr=0;         
                for tt = 1:len
                    if fret_smth(tt)<.05 || fret_smth(tt)>.95
                        count=count+1;
                    else
                        hist_ctr=tt-count;
                        ft(hist_ctr)=fret_smth(tt);          
                    end
                end
                frt=[0 1];
                for hh=1:hist_ctr
                    frt(hh+2)=ft(hh);
                end
      
                histogram=[hist(frt,50) zeros(1,len-50)];

                % Detecting edges using the Gaussian kernel.
                if result0{2} == 'Y'
                    disp(['      Detecting edges using the Gaussian kernel.'])
                    thresholds = str2num(result2{1});
                    scales = str2num(result2{2});
                    tranrad = str2num(result2{3});
                    [GKtransitions, GKtransindex] = GKedgedetection(donor_1, acceptor_1, fret_smth, tranrad, starttime, endtime, timestep, thresholds, deriv, scales, width);
                end
                
                % Detecting edges using the Chung-Kennedy filter.
                if result0{3} == 'Y'
                    disp(['      Detecting edges using the Chung-Kennedy filter.'])
                    CKW = str2num(result3{1});
                    PCT = str2num(result3{2});
                    SEP = str2num(result3{3});
                    if mod(CKW,2) == 1
                        CKW = CKW+1;
                    end
                    [CKtransitions, CKtransindex] = CKedgedetection(donor_1, acceptor_1, fret_smth, CKW, PCT, SEP, starttime, timestep);
                end
                
                
                % Aligning the results of the two edge detection methods weighted by how many times that method detected the transition.
                disp(['      Aligning the results of the edge detection methods.'])
                endtp = str2num(num2str(len*timestep));
                [GKtrans_aligned, CKtrans_aligned, output] = align_extract_trans_data(donor_1, acceptor_1, fret_smth, GKtransindex, CKtransindex, GKtransitions, CKtransitions, starttime, endtp, timestep);
                
                % Saving the data to an "all_traces" folder.
                disp(['      Saving the data to the all_traces folder.'])
                dir2 = 'all_traces/';
                cd([directory dir2])  
       
                % Writing out the column headings
                if molnumber < 10
                    fname=[file 'trace00' num2str(molnumber) '.txt'];
                elseif molnumber > 99
                    fname=[file 'trace' num2str(molnumber) '.txt'];
                else
                    fname=[file 'trace0' num2str(molnumber) '.txt'];
                end
                casctrace=['casc' num2str(movienum) 'trc' num2str(molnumber)];
                [fid2,~] = fopen(fname,'w');
                fprintf(fid2,'Donor\t');
                fprintf(fid2,casctrace);
                fprintf(fid2,'Acceptor\t');
                fprintf(fid2,casctrace);
                fprintf(fid2,'DonorRaw\t');
                fprintf(fid2,casctrace);
                fprintf(fid2,'AcceptorRaw\t');                
                fprintf(fid2,casctrace);
                fprintf(fid2,'FRET\t');
                fprintf(fid2,casctrace);
                fprintf(fid2,'FRETRaw\t');
                fprintf(fid2,casctrace);
                fprintf(fid2,'FRETRes\t');
                fprintf(fid2,casctrace);
                fprintf(fid2,'FRETHist\t');
                fprintf(fid2,casctrace); 
                fprintf(fid2,'BadData\t');
                fprintf(fid2,casctrace); 
                fprintf(fid2,'GKTransitions\t');
                fprintf(fid2,casctrace); 
                fprintf(fid2,'CKTransitions\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'FRETAvg\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'FRETStd\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'DonorAvg\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'AcceptorAvg\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'DonorStd\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'AcceptorStd\t');
                fprintf(fid2,casctrace); 
                fprintf(fid2,'FRETUpperCI\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'FRETLowerCI\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'DonorUpperCI\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'DonorLowerCI\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'AcceptorUpperCI\t');
                fprintf(fid2,casctrace);  
                fprintf(fid2,'AcceptorLowerCI');
                fprintf(fid2,casctrace); 
                
                fprintf(fid2,'\n');
                   
                if length(fret_smth) < length(donor_1)
                    lent = length(fret_smth);
                else lent = length(acceptor_1);
                end
        
                % Writing out the data.
                for i = 1:lent
                    fprintf(fid2, num2str(donor_1(i,1)));
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(acceptor_1(i,1)));
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(donorraw(i,1)));
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(acceptorraw(i,1)));
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(fret_smth(i,1)));
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(fretraw(i,1)));
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(fretres(i)));
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(histogram(i)));
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(databad(i))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(GKtrans_aligned(i)));  
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(CKtrans_aligned(i))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,1))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,2))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,3))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,4))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,5))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,6))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,8))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,7))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,10))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,9))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,12))); 
                    fprintf(fid2, '\t');
                    fprintf(fid2, num2str(output(i,11)));                     
                    fprintf(fid2, '\n');
                end
            
                % Moving to the next molecule and getting back to the original folder.
                fclose(fid2); 
                molnumber=molnumber+1;   
                cd ../
            end
        end
    end 
end



